
/**
 CKCoreDeepgram.h
 
 This is the header file for the CKCoreDeepgram class, which is responsible for converting text to speech using the Deepgram API. It also defines various constants and typedefs used in the class.
 
 */

#ifndef CKCoreDeepgram_h
#define CKCoreDeepgram_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

/**
 CKCoreDeepgramStreamCallback
 
 This typedef defines a block that is used as a callback for the streaming function of the Deepgram API. It takes in a NSURLSessionDataTask, NSData, and NSError as parameters.
 */
typedef void (^CKCoreDeepgramStreamCallback) (NSURLSessionDataTask *task, NSData *data, NSError *error);

//----------------------------------------------------------------------------------------------------
@interface CKCoreDeepgram: NSObject

/**
 text2SpeechToMp3File:withVoiceName:withLanguage:withText:withFileUrl:completionBlock:
 
 This method converts the given text to speech and saves the result as an mp3 file at the specified file URL. It takes in an API key, voice name, language, text, file URL, and completion block as parameters.
 
 @param apiKey The API key used to access the Deepgram API.
 @param withVoiceName The name of the voice to use for the speech.
 @param withLanguage The language to use for the speech.
 @param withText The text to be converted to speech.
 @param withFileUrl The file URL where the mp3 file will be saved.
 @param completionBlock A block that will be called when the conversion is complete, passing in an NSError if there was an error.
 */
+ (void) text2SpeechToMp3File:(NSString *) apiKey withVoiceName:(NSString *) withVoiceName withLanguage:(NSString *) withLanguage withText:(NSString *) withText withFileUrl:(NSURL *) withFileUrl completionBlock:(void (^)(NSError *error)) completionBlock;

/**
 text2SpeechToFile:withVoiceModel:withVoiceName:withLanguage:withText:withEncoding:withSampleRate:withFileUrl:completionBlock:
 
 This method converts the given text to speech and saves the result as a file at the specified file URL. It takes in an API key, voice model, voice name, language, text, encoding, sample rate, file URL, and completion block as parameters.
 
 @param apiKey The API key used to access the Deepgram API.
 @param withVoiceModel The model of the voice to use for the speech.
 @param withVoiceName The name of the voice to use for the speech.
 @param withLanguage The language to use for the speech.
 @param withText The text to be converted to speech.
 @param withEncoding The encoding to use for the speech.
 @param withSampleRate The sample rate to use for the speech.
 @param withFileUrl The file URL where the speech file will be saved.
 @param completionBlock A block that will be called when the conversion is complete, passing in an NSError if there was an error.
 */
+ (void) text2SpeechToFile:(NSString *) apiKey withVoiceModel:(NSString *) withVoiceModel withVoiceName:(NSString *) withVoiceName withLanguage:(NSString *) withLanguage withText:(NSString *) withText withEncoding:(NSString *) withEncoding withSampleRate:(NSInteger) withSampleRate withFileUrl:(NSURL *) withFileUrl completionBlock:(void (^)(NSError *error)) completionBlock;

/**
 text2Speech:withVoiceModel:withVoiceName:withLanguage:withText:withEncoding:withSampleRate:completionBlock:
 
 This method converts the given text to speech and returns the result as an NSData object. It takes in an API key, voice model, voice name, language, text, encoding, sample rate, and completion block as parameters.
 
 @param apiKey The API key used to access the Deepgram API.
 @param withVoiceModel The model of the voice to use for the speech.
 @param withVoiceName The name of the voice to use for the speech.
 @param withLanguage The language to use for the speech.
 @param withText The text to be converted to speech.
 @param withEncoding The encoding to use for the speech.
 @param withSampleRate The sample rate to use for the speech.
 @param completionBlock A block that will be called when the conversion is complete, passing in the result data, status code, and an NSError if there was an error.
 */
+ (void) text2Speech:(NSString *) apiKey withVoiceModel:(NSString *) withVoiceModel withVoiceName:(NSString *) withVoiceName withLanguage:(NSString *) withLanguage withText:(NSString *) withText withEncoding:(NSString *) withEncoding withSampleRate:(NSInteger) withSampleRate completionBlock:(void (^)(NSData *result, NSInteger statusCode, NSError *error)) completionBlock;

@end

/**
 Deepgram Voice Models
 
 These constants define the different voice models available for use with the Deepgram API.
 */
#define DEEPGRAM_VOICEMODEL_AURA   @"aura"

/**
 Deepgram Voice Names
 
 These constants define the different voice names available for use with the Deepgram API.
 */
#define DEEPGRAM_VOICENAME_Asteria @"asteria"
#define DEEPGRAM_VOICENAME_Luna    @"luna"
#define DEEPGRAM_VOICENAME_Stella  @"stella"
#define DEEPGRAM_VOICENAME_Athena  @"athena"
#define DEEPGRAM_VOICENAME_Hera    @"hera"
#define DEEPGRAM_VOICENAME_Orion   @"orion"
#define DEEPGRAM_VOICENAME_Arcas   @"arcas"
#define DEEPGRAM_VOICENAME_Perseus @"perseus"
#define DEEPGRAM_VOICENAME_Angus   @"angus"
#define DEEPGRAM_VOICENAME_Orpheus @"orpheus"
#define DEEPGRAM_VOICENAME_Helios  @"helios"
#define DEEPGRAM_VOICENAME_Zeus    @"zeus"

/**
 Deepgram Encodings
 
 These constants define the different encodings available for use with the Deepgram API.
 */
#define DEEPGRAM_ENCODING_linear16 @"linear16"
#define DEEPGRAM_ENCODING_mulaw    @"mulaw"
#define DEEPGRAM_ENCODING_alaw     @"alaw"
#define DEEPGRAM_ENCODING_mp3      @"mp3"
#define DEEPGRAM_ENCODING_opus     @"opus"
#define DEEPGRAM_ENCODING_flac     @"flac"
#define DEEPGRAM_ENCODING_aac      @"aac"

/**
 Deepgram Sample Rates
 
 These constants define the different sample rates available for use with the Deepgram API.
 */
#define DEEPGRAM_SAMPLERATE_8000   8000
#define DEEPGRAM_SAMPLERATE_16000  16000
#define DEEPGRAM_SAMPLERATE_22050  22050
#define DEEPGRAM_SAMPLERATE_24000  24000
#define DEEPGRAM_SAMPLERATE_32000  32000
#define DEEPGRAM_SAMPLERATE_48000  48000

#endif
