#ifndef NSViewNotification_h
#define NSViewNotification_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

extern const NSString * NSViewNotificationPositionTop;
extern const NSString * NSViewNotificationPositionCenter;
extern const NSString * NSViewNotificationPositionBottom;

@class NSViewNotificationStyle;

/**
 notification is an Objective-C category that adds notification notifications to the UIView
 object class. It is intended to be simple, lightweight, and easy to use. Most
 notification notifications can be triggered with a single line of code.
 
 The `makeNotification:` methods create a new view and then display it as notification.
 
 The `shownotification:` methods display any view as notification.
 
 */
@interface UIView (Notification)

/**
 Creates and presents a new notification view with a message and displays it with the
 default duration and position. Styled using the shared style.
 
 @param message The message to be displayed
 */
- (void)makeNotification:(NSString *)message;

/**
 Creates and presents a new notification view with a message. Duration and position
 can be set explicitly. Styled using the shared style.
 
 @param message The message to be displayed
 @param duration The notification duration
 @param position The notification's center point. Can be one of the predefined NSViewNotificationPosition
                 constants or a `CGPoint` wrapped in an `NSValue` object.
 */
- (void)makeNotification:(NSString *)message
         duration:(NSTimeInterval)duration
         position:(id)position;

/**
 Creates and presents a new notification view with a message. Duration, position, and
 style can be set explicitly.
 
 @param message The message to be displayed
 @param duration The notification duration
 @param position The notification's center point. Can be one of the predefined NSViewNotificationPosition
 constants or a `CGPoint` wrapped in an `NSValue` object.
 @param style The style. The shared style will be used when nil
 */
- (void)makeNotification:(NSString *)message
         duration:(NSTimeInterval)duration
         position:(id)position
            style:(NSViewNotificationStyle *)style;

/**
 Creates and presents a new notification view with a message, title, and image. Duration,
 position, and style can be set explicitly. The completion block executes when the
 notification view completes. `didTap` will be `YES` if the notification view was dismissed from 
 a tap.
 
 @param message The message to be displayed
 @param duration The notification duration
 @param position The notification's center point. Can be one of the predefined NSViewNotificationPosition
                 constants or a `CGPoint` wrapped in an `NSValue` object.
 @param title The title
 @param image The image
 @param style The style. The shared style will be used when nil
 @param completion The completion block, executed after the notification view disappears.
                   didTap will be `YES` if the notification view was dismissed from a tap.
 */
- (void)makeNotification:(NSString *)message
         duration:(NSTimeInterval)duration
         position:(id)position
            title:(NSString *)title
            image:(UIImage *)image
            style:(NSViewNotificationStyle *)style
       completion:(void(^)(BOOL didTap))completion;

/**
 Creates a new notification view with any combination of message, title, and image.
 The look and feel is configured via the style. Unlike the `makeNotification:` methods,
 this method does not present the notification view automatically. One of the shownotification:
 methods must be used to present the resulting view.
 
 @warning if message, title, and image are all nil, this method will return nil.
 
 @param message The message to be displayed
 @param title The title
 @param image The image
 @param style The style. The shared style will be used when nil
 @return The newly created notification view
 */
- (UIView *)notificationViewForMessage:(NSString *)message
                          title:(NSString *)title
                          image:(UIImage *)image
                          style:(NSViewNotificationStyle *)style;

/**
 Hides the active notification. If there are multiple notifications active in a view, this method
 hides the oldest notification (the first of the notifications to have been presented).
 
 @see `hideAllnotifications` to remove all active notifications from a view.
 
 @warning This method has no effect on activity notifications. Use `hidenotificationActivity` to
 hide activity notifications.
 */
- (void)hideNotification;

/**
 Hides an active Notification.
 
 @param notification The active Notification view to dismiss. Any Notification that is currently being displayed
 on the screen is considered active.
 
 @warning this does not clear a notification view that is currently waiting in the queue.
 */
- (void)hideNotification:(UIView *)notification;

/**
 Hides all active notification views and clears the queue.
 */
- (void)hideAllNotifications;

/**
 Hides all active notification views, with options to hide activity and clear the queue.
 
 @param includeActivity If `true`, notification activity will also be hidden. Default is `false`.
 @param clearQueue If `true`, removes all notification views from the queue. Default is `true`.
 */
- (void)hideAllNotifications:(BOOL)includeActivity clearQueue:(BOOL)clearQueue;

/**
 Removes all notification views from the queue. This has no effect on notification views that are
 active. Use `hideAllnotifications` to hide the active notifications views and clear the queue.
 */
- (void)clearNotificationQueue;

/**
 Creates and displays a new notification activity indicator view at a specified position.
 
 @warning Only one notification activity indicator view can be presented per superview. Subsequent
 calls to `makeNotificationActivity:` will be ignored until hidenotificationActivity is called.
 
 @warning `makeNotificationActivity:` works independently of the shownotification: methods. notification activity
 views can be presented and dismissed while notification views are being displayed. `makeNotificationActivity:`
 has no effect on the queueing behavior of the shownotification: methods.
 
 @param position The notification's center point. Can be one of the predefined NSViewNotificationPosition
                 constants or a `CGPoint` wrapped in an `NSValue` object.
 */
- (void)makeNotificationActivity:(id)position;

/**
 Dismisses the active notification activity indicator view.
 */
- (void)hideNotificationActivity;

/**
 Displays any view as notification using the default duration and position.
 
 @param notification The view to be displayed as notification
 */
- (void)showNotification:(UIView *)notification;

/**
 Displays any view as notification at a provided position and duration. The completion block 
 executes when the notification view completes. `didTap` will be `YES` if the notification view was 
 dismissed from a tap.
 
 @param notification The view to be displayed as notification
 @param duration The notification duration
 @param position The notification's center point. Can be one of the predefined NSViewNotificationPosition
                 constants or a `CGPoint` wrapped in an `NSValue` object.
 @param completion The completion block, executed after the notification view disappears.
                   didTap will be `YES` if the notification view was dismissed from a tap.
 */
- (void)showNotification:(UIView *)notification
         duration:(NSTimeInterval)duration
         position:(id)position
       completion:(void(^)(BOOL didTap))completion;

@end

/**
 `NSViewNotificationStyle` instances define the look and feel for notification views created via the 
 `makeNotification:` methods as well for notification views created directly with
 `notificationViewForMessage:title:image:style:`.
 
 @warning `NSViewNotificationStyle` offers relatively simple styling options for the default
 notification view. If you require a notification view with more complex UI, it probably makes more
 sense to create your own custom UIView subclass and present it with the `shownotification:`
 methods.
 */
@interface NSViewNotificationStyle : NSObject

/**
 The background color. Default is `[UIColor blackColor]` at 80% opacity.
 */
@property (strong, nonatomic) UIColor *backgroundColor;

/**
 The title color. Default is `[UIColor whiteColor]`.
 */
@property (strong, nonatomic) UIColor *titleColor;

/**
 The message color. Default is `[UIColor whiteColor]`.
 */
@property (strong, nonatomic) UIColor *messageColor;

/**
 A percentage value from 0.0 to 1.0, representing the maximum width of the notification
 view relative to it's superview. Default is 0.8 (80% of the superview's width).
 */
@property (assign, nonatomic) CGFloat maxWidthPercentage;

/**
 A percentage value from 0.0 to 1.0, representing the maximum height of the notification
 view relative to it's superview. Default is 0.8 (80% of the superview's height).
 */
@property (assign, nonatomic) CGFloat maxHeightPercentage;

/**
 The spacing from the horizontal edge of the notification view to the content. When an image
 is present, this is also used as the padding between the image and the text.
 Default is 10.0.
 */
@property (assign, nonatomic) CGFloat horizontalPadding;

/**
 The spacing from the vertical edge of the notification view to the content. When a title
 is present, this is also used as the padding between the title and the message.
 Default is 10.0.
 */
@property (assign, nonatomic) CGFloat verticalPadding;

/**
 The corner radius. Default is 10.0.
 */
@property (assign, nonatomic) CGFloat cornerRadius;

/**
 The title font. Default is `[UIFont boldSystemFontOfSize:16.0]`.
 */
@property (strong, nonatomic) UIFont *titleFont;

/**
 The message font. Default is `[UIFont systemFontOfSize:16.0]`.
 */
@property (strong, nonatomic) UIFont *messageFont;

/**
 The title text alignment. Default is `NSTextAlignmentLeft`.
 */
@property (assign, nonatomic) NSTextAlignment titleAlignment;

/**
 The message text alignment. Default is `NSTextAlignmentLeft`.
 */
@property (assign, nonatomic) NSTextAlignment messageAlignment;

/**
 The maximum number of lines for the title. The default is 0 (no limit).
 */
@property (assign, nonatomic) NSInteger titleNumberOfLines;

/**
 The maximum number of lines for the message. The default is 0 (no limit).
 */
@property (assign, nonatomic) NSInteger messageNumberOfLines;

/**
 Enable or disable a shadow on the notification view. Default is `NO`.
 */
@property (assign, nonatomic) BOOL displayShadow;

/**
 The shadow color. Default is `[UIColor blackColor]`.
 */
@property (strong, nonatomic) UIColor *shadowColor;

/**
 A value from 0.0 to 1.0, representing the opacity of the shadow.
 Default is 0.8 (80% opacity).
 */
@property (assign, nonatomic) CGFloat shadowOpacity;

/**
 The shadow radius. Default is 6.0.
 */
@property (assign, nonatomic) CGFloat shadowRadius;

/**
 The shadow offset. The default is `CGSizeMake(4.0, 4.0)`.
 */
@property (assign, nonatomic) CGSize shadowOffset;

/**
 The image size. The default is `CGSizeMake(80.0, 80.0)`.
 */
@property (assign, nonatomic) CGSize imageSize;

/**
 The size of the notification activity view when `makeNotificationActivity:` is called.
 Default is `CGSizeMake(100.0, 100.0)`.
 */
@property (assign, nonatomic) CGSize activitySize;

/**
 The fade in/out animation duration. Default is 0.2.
 */
@property (assign, nonatomic) NSTimeInterval fadeDuration;

/**
 Creates a new instance of `NSViewNotificationStyle` with all the default values set.
 */
- (instancetype)initWithDefaultStyle NS_DESIGNATED_INITIALIZER;

/**
 @warning Only the designated initializer should be used to create
 an instance of `NSViewNotificationStyle`.
 */
- (instancetype)init NS_UNAVAILABLE;

@end

/**
 `NSViewNotificationManager` provides general configuration options for all notification
 notifications. Backed by a singleton instance.
 */
@interface NSViewNotificationManager : NSObject

/**
 Sets the shared style on the singleton. The shared style is used whenever
 a `makeNotification:` method (or `notificationViewForMessage:title:image:style:`) is called
 with with a nil style. By default, this is set to `NSViewNotificationStyle`'s default
 style.
 
 @param sharedStyle the shared style
 */
+ (void)setSharedStyle:(NSViewNotificationStyle *)sharedStyle;

/**
 Gets the shared style from the singlton. By default, this is
 `NSViewNotificationStyle`'s default style.
 
 @return the shared style
 */
+ (NSViewNotificationStyle *)sharedStyle;

/**
 Enables or disables tap to dismiss on notification views. Default is `YES`.
 
 @param tapToDismissEnabled YES or NO
 */
+ (void)setTapToDismissEnabled:(BOOL)tapToDismissEnabled;

/**
 Returns `YES` if tap to dismiss is enabled, otherwise `NO`.
 Default is `YES`.
 
 @return BOOL YES or NO
 */
+ (BOOL)isTapToDismissEnabled;

/**
 Enables or disables queueing behavior for notification views. When `YES`,
 notification views will appear one after the other. When `NO`, multiple notification
 views will appear at the same time (potentially overlapping depending
 on their positions). This has no effect on the notification activity view,
 which operates independently of normal notification views. Default is `NO`.
 
 @param queueEnabled YES or NO
 */
+ (void)setQueueEnabled:(BOOL)queueEnabled;

/**
 Returns `YES` if the queue is enabled, otherwise `NO`.
 Default is `NO`.
 
 @return BOOL
 */
+ (BOOL)isQueueEnabled;

/**
 Sets the default duration. Used for the `makeNotification:` and
 `shownotification:` methods that don't require an explicit duration.
 Default is 3.0.
 
 @param duration The notification duration
 */
+ (void)setDefaultDuration:(NSTimeInterval)duration;

/**
 Returns the default duration. Default is 3.0.
 
 @return duration The notification duration
*/
+ (NSTimeInterval)defaultDuration;

/**
 Sets the default position. Used for the `makeNotification:` and
 `shownotification:` methods that don't require an explicit position.
 Default is `NSViewNotificationPositionBottom`.
 
 @param position The default center point. Can be one of the predefined
 NSViewNotificationPosition constants or a `CGPoint` wrapped in an `NSValue` object.
 */
+ (void) setDefaultPosition:(id) position;

/**
 Returns the default notification position. Default is `NSViewNotificationPositionBottom`.
 
 @return position The default center point. Will be one of the predefined
 NSViewNotificationPosition constants or a `CGPoint` wrapped in an `NSValue` object.
 */
+ (id) defaultPosition;

@end

#endif

